﻿/*
*	FILE			: BLEManager.cs
*	PROJECT			: Black Box
*	PROGRAMMER		: BRIAN HINDS
*	FIRST VERSION	: 1.0.0.0
*	DATE            : March 20, 2019
*	DESCRIPTION 	: Used for manage and discover Bluetooth LE device
*/

using System;
using System.Collections.Generic;
using Windows.Devices.Bluetooth;
using Windows.Devices.Bluetooth.GenericAttributeProfile;
using Windows.Devices.Enumeration;
using Windows.Storage.Streams;

namespace MiBand_Heartrate
{
    /* BLEManager
     * Used for manage and discover Bluetooth LE device
     * see https://docs.microsoft.com/en-us/windows/uwp/devices-sensors/gatt-client
     */

    internal class BLEManager
    {
        public delegate void OnDeviceConnected(BLEManager manager);

        private DeviceWatcher watcher = null;
        private List<DeviceInformation> devices = new List<DeviceInformation>();
        private BluetoothLEDevice connectedDevice = null;

        public BluetoothLEDevice Device => connectedDevice;

        /// <summary>
        /// Manages fitness tracker
        /// </summary>
        /// <returns></returns>
        public DeviceWatcher CreateWatcher()
        {
            if (watcher == null)
            {
                devices.Clear();
                string[] requestedProperties = { "System.Devices.Aep.DeviceAddress", "System.Devices.Aep.IsConnected" };
                watcher = DeviceInformation.CreateWatcher(
                    BluetoothLEDevice.GetDeviceSelectorFromPairingState(true),
                    requestedProperties,
                    DeviceInformationKind.AssociationEndpoint
                );

                watcher.Added += (DeviceWatcher w, DeviceInformation d) =>
                {
                    devices.Add(d);
                };

                watcher.Updated += (DeviceWatcher w, DeviceInformationUpdate d) =>
                {
                    foreach (DeviceInformation device in devices)
                    {
                        if (device.Id == d.Id) { device.Update(d); break; }
                    }
                };

                watcher.Removed += (DeviceWatcher w, DeviceInformationUpdate d) =>
                {
                    foreach (DeviceInformation device in devices)
                    {
                        if (device.Id == d.Id) { devices.Remove(device); break; }
                    }
                };
            }

            return watcher;
        }

        public void StartWatcher()
        {
            watcher.Start();
        }

        public void StopWatcher()
        {
            watcher.Stop();
        }

        /// <summary>
        /// </summary>
        /// <param name="deviceId"></param>
        /// <param name="callback"></param>
        public async void Connect(string deviceId, OnDeviceConnected callback = null)
        {
            connectedDevice = await BluetoothLEDevice.FromIdAsync(deviceId);
            callback.Invoke(this);
        }

        /// <summary>
        /// </summary>
        public void Disconnect()
        {
            if (connectedDevice != null) { connectedDevice.Dispose(); connectedDevice = null; }
        }

        /// <summary>
        /// </summary>
        /// <returns></returns>
        public bool IsWatchingForDevice()
        {
            return (watcher != null && watcher.Status == DeviceWatcherStatus.Started);
        }

        /// <summary>
        /// </summary>
        /// <returns></returns>
        public bool HasDeviceConnected()
        {
            return (connectedDevice != null);
        }

        /// <summary>
        /// </summary>
        /// <param name="c"></param>
        /// <param name="data"></param>
        public async void Write(GattCharacteristic c, byte[] data)
        {
            if (!HasDeviceConnected()) { return; }

            using (DataWriter stream = new DataWriter())
            {
                stream.WriteBytes(data);

                try
                {
                    GattCommunicationStatus result = await c.WriteValueAsync(stream.DetachBuffer());
                    if (result != GattCommunicationStatus.Success)
                    {
                        Console.WriteLine(string.Format("Write {0} on {1} failed !", BitConverter.ToString(data), c.Uuid));
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                }
            }
        }

        /// <summary>
        /// Not used but can be useful for debugging
        /// </summary>
        public async void EnumeratingUsages()
        {
            if (!HasDeviceConnected()) { return; }

            GattDeviceServicesResult services = await connectedDevice.GetGattServicesAsync();
            if (services.Status == GattCommunicationStatus.Success)
            {
                foreach (GattDeviceService s in services.Services)
                {
                    await s.RequestAccessAsync();
                    GattCharacteristicsResult characteristicsResult = await s.GetCharacteristicsAsync();

                    Console.WriteLine("Service : " + s.Uuid);
                    if (characteristicsResult.Status == GattCommunicationStatus.Success)
                    {
                        foreach (GattCharacteristic c in characteristicsResult.Characteristics)
                        {
                            GattCharacteristicProperties props = c.CharacteristicProperties;
                            Console.WriteLine("\t characteristics : " + c.Uuid + " / " + c.UserDescription);
                            if (props.HasFlag(GattCharacteristicProperties.Read))
                            {
                                Console.WriteLine("\t\tRead");
                            }
                            if (props.HasFlag(GattCharacteristicProperties.Write))
                            {
                                Console.WriteLine("\t\tWrite");
                            }
                            if (props.HasFlag(GattCharacteristicProperties.Notify))
                            {
                                Console.WriteLine("\t\tNotify");
                            }
                        }
                    }

                    s.Dispose();
                }
            }
        }
    }
}