﻿/*
*	FILE			: program.cs
*	PROJECT			: Black Box
*	PROGRAMMER		: BRIAN HINDS
*	FIRST VERSION	: 1.0.0.0
*	DATE            : March 20, 2019
*	DESCRIPTION 	: This file is the main program for the BBPresenseSensor.
*/

using BBSensorConnection;
using BBSensorsErrorLog;
using BBTaskerThermostatStatus;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Net.Sockets;
using System.Threading;

/*
 *
 * Presence Sensor Architecture
 *
 *
Conestoga College

Copyright (c) 2019, Brian Hinds

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sub-license, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

namespace BBPresenseSensor
{
    internal class Program
    {
        #region Variables

        // Connection string needed for the connection library
        private static readonly string BBconection = ConfigurationManager.ConnectionStrings["BB_Sensor_CONNECTION"].ConnectionString;

        // Client Collection
        private static List<BBSensorObjects> _sensors;

        // Thread Signal
        private static ManualResetEvent _connected = new ManualResetEvent(false);

        //private static BBPresenceSensorStatus bbs;

        public static bool isItRunning = false;

        // Server socket
        private static Socket _sensorServer = null;

        #endregion Variables

        #region Main

        private static void Main(string[] args)
        {
            BBSensorConnectionManager.parseConnectionData(BBconection);
            Console.Title = "BBPresenceSensor";

            _sensors = new List<BBSensorObjects>();
            StartListening();
            Console.ReadLine();
            CloseAllSockets();
        }

        #endregion Main

        #region Listener

        /// <summary>
        /// Listen for client connections
        /// </summary>
        private static void StartListening()
        {
            try
            {
                _sensorServer = BBSensorConnectionManager.CreateListener();

                while (true)
                {
                    // Set the event to non signaled state
                    _connected.Reset();

                    // Start an asynchronous socket to listen for connections
                    _sensorServer.BeginAccept(new AsyncCallback(AcceptCallback), _sensorServer);

                    // Set operation status to true
                    BBThermostatStatus.SensorPStatus(true);

                    // Wait until a connection is made before continuing
                    _connected.WaitOne();
                }
            }
            catch (Exception ex)
            {
                BBSensorErrorLog.SaveErrorData("PIR Sensor " + ex.Message.ToString());
            }
        }

        #endregion Listener

        #region AcceptCallback

        /// <summary>
        /// Handler for new connections
        /// </summary>
        /// <param name="ar"></param>
        private static void AcceptCallback(IAsyncResult ar)
        {
            // Signal the main thread to continue accepting new connections
            _connected.Set();

            // Accept new client socket connection
            Socket socket = _sensorServer.EndAccept(ar);

            // Create a new client connection object and store the socket
            BBSensorObjects client = new BBSensorObjects
            {
                Socket = socket
            };

            // Store all clients
            _sensors.Add(client);

            // Begin receiving messages from new connection
            try
            {
                client.Socket.BeginReceive(client.Buffer, 0, client.BufferSize, SocketFlags.None, new AsyncCallback(ReceiveCallback), client);
            }
            catch (SocketException)
            {
                // Client was forcibly closed on the client side
                CloseClient(client);
            }
            catch (Exception ex)
            {
                // Set operation status to true

                BBSensorErrorLog.SaveErrorData("PIR Sensor " + ex.Message.ToString());
            }
        }

        #endregion AcceptCallback

        #region ReceiveCallback

        /// <summary>
        /// Handler for received messages
        /// </summary>
        /// <param name="ar"></param>
        private static void ReceiveCallback(IAsyncResult ar)
        {
            int bytesRead;

            // Check for null values
            if (!CheckState(ar, out string err, out BBSensorObjects client))
            {
                BBSensorErrorLog.SaveErrorData(err);

                return;
            }

            // Read message from the client socket
            try
            {
                bytesRead = client.Socket.EndReceive(ar);
            }
            catch (SocketException)
            {
                // Client was forcedly closed on the client side
                CloseClient(client);

                return;
            }
            catch (Exception ex)
            {
                BBSensorErrorLog.SaveErrorData("PIR Sensor " + ex.Message.ToString());

                return;
            }

            // Check message
            if (bytesRead > 0)
            {
                // Build message as it comes in
                client.BuildIncomingMessage(bytesRead);

                // Print message to the console
                client.PrintMessage();

                // Reset message
                client.ClearIncomingMessage();
            }

            // Listen for more incoming messages
            try
            {
                client.Socket.BeginReceive(client.Buffer, 0, client.BufferSize, SocketFlags.None, new AsyncCallback(ReceiveCallback), client);
            }
            catch (SocketException)
            {
                // Client was forcedly closed on the client side
                CloseClient(client);
            }
            catch (Exception ex)
            {
                BBSensorErrorLog.SaveErrorData("PIR Sensor " + ex.Message.ToString());
            }
        }

        #endregion ReceiveCallback

        #region CheckState

        /// <summary>
        /// Checks IAsyncResult for null value
        /// </summary>
        /// <param name="ar"></param>
        /// <param name="err"></param>
        /// <param name="client"></param>
        /// <returns></returns>
        private static bool CheckState(IAsyncResult ar, out string err, out BBSensorObjects client)
        {
            // Initialize
            client = null;
            err = "";

            // Check ar
            if (ar == null)
            {
                err = "Async result null";

                return false;
            }

            // Check client
            client = (BBSensorObjects)ar.AsyncState;
            if (client == null)
            {
                err = "Client null";

                return false;
            }

            return true;
        }

        #endregion CheckState

        #region CloseClient

        /// <summary>
        /// Closes a client socket and removes it from the client list
        /// </summary>
        /// <param name="client"></param>
        private static void CloseClient(BBSensorObjects client)
        {
            BBSensorErrorLog.SaveErrorData("Client disconnected");
            client.Close();
            if (_sensors.Contains(client))
            {
                _sensors.Remove(client);
            }

            //Console.WriteLine(_sensors.Count().ToString());

            foreach (BBSensorObjects sensor in _sensors)
            {
                Console.WriteLine(sensor);
            }
        }

        #endregion CloseClient

        #region CloseAllSockets

        /// <summary>
        /// Closes all client and server connections
        /// </summary>
        private static void CloseAllSockets()
        {
            // Close all clients
            foreach (BBSensorObjects connection in _sensors)
            {
                connection.Close();
            }

            // Close server
            _sensorServer.Close();
        }

        #endregion CloseAllSockets
    }
}