﻿/*
*	FILE			: BBSensorObjects.cs
*	PROJECT			: Black Box
*	PROGRAMMER		: BRIAN HINDS
*	FIRST VERSION	: 1.0.0.0
*	DATE            : March 20, 2019
*	DESCRIPTION 	: This file manages the connection object, incoming message and outgoing message.
*/

using BBSensorsErrorLog;
using System;
using System.Net.Sockets;
using System.Text;
using System.Text.RegularExpressions;

namespace BBSensorConnection
{
    public class BBSensorObjects
    {
        #region Properties

        // boolean for movement
        private static bool isMovement = true;

        public bool move;

        // boolean for no movement
        private static bool noMovement = false;

        // Raw data from presence data
        private static string presenseData = "";

        // Location of presence sensor
        private static string location = "";

        // Status of presence sensor
        private static string status = "";

        // current temperature upon movement
        private static string temperature = "";

        // current humidity upon movement
        private static string humidity = "";

        // current temperature upon movement
        private static string temperatureBuffer = "0.0";

        // current humidity upon movement
        private static string humidityBuffer = "0.0";

        // Client socket
        public Socket Socket { get; set; }

        // Size of receive buffer
        public int BufferSize { get; set; } = 256;

        // Receive buffer
        public byte[] Buffer { get; set; }

        // Received data string
        private StringBuilder IncomingMessage { get; set; }

        // Message to be sent
        private StringBuilder OutgoingMessage { get; set; }

        // Terminator for each message
        public string MessageTerminator { get; set; } = "<END>";

        public bool isRunning { get; set; }

        #endregion Properties

        #region Constructors

        public BBSensorObjects()
        {
            Buffer = new byte[BufferSize];
            IncomingMessage = new StringBuilder();
            OutgoingMessage = new StringBuilder();
            isRunning = true;
        }

        #endregion Constructors

        #region Outgoing Message Methods

        /// <summary>
        /// Converts the outgoing message to bytes
        /// </summary>
        /// <returns></returns>
        public byte[] OutgoingMessageToBytes()
        {
            if (OutgoingMessage.ToString().IndexOf(MessageTerminator) < 0)
            {
                OutgoingMessage.Append(MessageTerminator);
            }
            return Encoding.ASCII.GetBytes(OutgoingMessage.ToString());
        }

        /// <summary>
        /// Creates a new outgoing message
        /// </summary>
        /// <param name="msg"></param>
        public void CreateOutgoingMessage(string msg)
        {
            OutgoingMessage.Clear();
            OutgoingMessage.Append(msg);
            OutgoingMessage.Append(MessageTerminator);
        }

        #endregion Outgoing Message Methods

        #region Incoming Message Methods

        /// <summary>
        /// Converts the buffer to a string ans stores it
        /// </summary>
        public void BuildIncomingMessage(int bytesRead)
        {
            IncomingMessage.Append(Encoding.ASCII.GetString(Buffer, 0, bytesRead));
            //Console.WriteLine(IncomingMessage.Append(Encoding.ASCII.GetString(Buffer, 0, bytesRead)));
        }

        /// <summary>
        /// Determines if the message was fully received
        /// </summary>
        /// <returns></returns>
        public bool MessageReceived()
        {
            return IncomingMessage.ToString().IndexOf(MessageTerminator) > -1;
        }

        /// <summary>
        /// Clears the current incoming message so that we can start building for the next message
        /// </summary>
        public void ClearIncomingMessage()
        {
            IncomingMessage.Clear();
        }

        /// <summary>
        /// Gets the length of the incoming message
        /// </summary>
        /// <returns></returns>
        public int IncomingMessageLength()
        {
            return IncomingMessage.Length;
        }

        #endregion Incoming Message Methods

        #region Connected Object Methods

        /// <summary>
        /// Closes the connection
        /// </summary>
        public void Close()
        {
            try
            {
                Socket.Shutdown(SocketShutdown.Both);
                Socket.Close();
            }
            catch (Exception ex)
            {
                BBSensorErrorLog.SaveErrorData("PIR Sensor connection already closed " + ex.Message.ToString());
            }
        }

        public string GetRemoteEndPoint()
        {
            return Socket.RemoteEndPoint.ToString();
        }

        /// <summary>
        /// Print the details of the current incoming message
        /// </summary>
        public void PrintMessage()
        {
            if (IncomingMessage.ToString().Contains("START") && isMovement)
            {
                parseData(IncomingMessage.ToString());

                isMovement = false;
                noMovement = true;
            }
            if (IncomingMessage.ToString().Contains("STOP") && noMovement)
            {
                parseData(IncomingMessage.ToString());
                isMovement = true;
                noMovement = false;
            }
        }

        #endregion Connected Object Methods

        #region Sensor Data Processing

        /// <summary>
        /// </summary>
        /// <param name="data"></param>
        private void parseData(string data)
        {
            try
            {
                presenseData = Regex.Replace(data, @"<[^>]+>|&nbsp;", "");

                int count = 0;

                string[] words = presenseData.Split(',');
                foreach (string word in words)
                {
                    if (count == 0) { status = word; }
                    if (count == 1) { location = word; }
                    if (count == 2) { temperature = word; }
                    if (count == 3) { humidity = word; }

                    count++;
                }

                if (temperature == "nan") { temperature = temperatureBuffer; }
                else { temperatureBuffer = temperature; }
                if (humidity == "nan") { humidity = humidityBuffer; }
                else { humidityBuffer = humidity; }

                // Debug window
                Console.Write(location + "," + status + "," + temperature + "," + humidity + DateTime.Now.ToString(" HH:mm:ss") + "," + DateTime.Now.ToString("yyyy-MM-dd") + Environment.NewLine);

                // Encrypt data
                BBSensorDataEncryptor.BBSensorDataEncryption.EncryptData(location + "," + status + "," + temperature + "," + humidity + "," + DateTime.Now.ToString("HH:mm:ss") + "," + DateTime.Now.ToString("yyyy-MM-dd"));
            }
            catch (Exception ex)
            {
                isRunning = false;
                BBSensorErrorLog.SaveErrorData("PIR Sensor " + ex.Message.ToString());
            }
        }

        #endregion Sensor Data Processing
    }
}