﻿/*
* FILE            : BBtaskScheduler.cs
* PROJECT         : BBTasker
* PROGRAMMER      : Brian Hinds
* DATE            : March 20, 2019
* FIRST VERSION   : 1.2.0
* DESCRIPTION     :Task Scheduler for the Black Box Architecture
*/

using BBSensorConnection;
using BBSensorsErrorLog;
using System;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows.Forms;
using Timer = System.Windows.Forms.Timer;

namespace BBTasker
{
    public partial class BBtaskScheduler : Form
    {
        #region Variables

        private Process PIRSensorTask = new Process();
        private Process ThermostatTask = new Process();
        private Process FitnessTask = new Process();
        private Process BBNetwork = new Process();
        private string BBNSetup = @"netSetup.bat";
        private string BBPpath = @"BBPresenseSensor.exe";
        private string BBTpath = @"BBThermostatSensor.exe";
        private string BBFpath = @"MiBand-Heartrate.exe";
        private BBSensorObjects BBSO = new BBSensorObjects();

        public static string StartDate = "";
        public static string StopDate = "";

        /// <summary>
        /// </summary>
        private bool BBPSStatus = false;

        /// <summary>
        /// </summary>
        private bool BBTSStatus = false;

        private string BBTSname = "";
        private bool BBTSGetModel = false;

        /// <summary>
        /// </summary>
        private bool BBFSStatus = false;

        private string fileName = "";

        private bool ScheduleSet = false;
        private DateTime startdate;
        private DateTime enddate;
        private DateTime today = DateTime.Today;

        #endregion Variables

        #region Constructor

        /// <summary>
        /// </summary>
        public BBtaskScheduler()
        {
            InitializeComponent();

            StatusChecker = new Timer();
            StatusChecker.Tick += new EventHandler(StatusChecker_Tick);
            StatusChecker.Interval = 5000;
            StatusChecker.Enabled = true;
            StatusChecker.Stop();
        }

        #endregion Constructor

        #region PluginProcessCheck

        /// <summary>
        /// Black Box presence sensor status
        /// </summary>
        private void BBPSS()
        {
            fileName = Path.GetFileName(BBPpath);

            //check if process still running
            Process[] processName = Process.GetProcessesByName(fileName.Substring(0, fileName.LastIndexOf('.')));
            if (processName.Length > 0)
            {
                MotionStatusIcon.Image = Properties.Resources.trafficlight_green;

                // Check running condition
                BBPSStatus = BBSO.isRunning;

                // Operation status
                if (BBPSStatus)
                {
                    MotionStatusIcon.Image = Properties.Resources.trafficlight_green;
                }
                if (!BBPSStatus)
                {
                    MotionStatusIcon.Image = Properties.Resources.trafficlight_yellow;
                }
            }
            else
            {
                MotionStatusIcon.Image = Properties.Resources.trafficlight_red;
                BBPSStatus = false;
            }
        }

        /// <summary>
        /// Black Box thermostat sensor status
        /// </summary>
        private void BBTSS()

        {
            string fileName = Path.GetFileName(BBTpath);

            //check if process still running
            Process[] processName = Process.GetProcessesByName(fileName.Substring(0, fileName.LastIndexOf('.')));
            if (processName.Length > 0)
            {
                ThermostatStatusIcon.Image = Properties.Resources.trafficlight_green;

                // Get thermostat model
                if (!BBTSGetModel)
                {
                    BBTSname = BBThermostatConnection.BBThermostatConnect.BBThermostatType();

                    BBTSname = RemoveUnwantedChar(BBTSname);

                    BBTModel.Text = BBTSname;
                    BBTSGetModel = true;
                }

                //Checking thermostat data status
                BBTSStatus = BBTaskerThermostatStatus.BBThermostatStatus.isRunning;

                // Operation status
                if (BBTSStatus)
                {
                    ThermostatStatusIcon.Image = Properties.Resources.trafficlight_green;
                }
                if (!BBTSStatus)
                {
                    ThermostatStatusIcon.Image = Properties.Resources.trafficlight_red;
                }
            }
            else
            {
                ThermostatStatusIcon.Image = Properties.Resources.trafficlight_red;
                BBTSStatus = false;
            }
        }

        /// <summary>
        /// Parse model info
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public string RemoveUnwantedChar(string input)
        {
            StringBuilder stringBuilder = new StringBuilder();

            foreach (object match in Regex.Matches(input, "[0-9nu BRevCTkMGHzVs%\\-.]"))
            {
                stringBuilder.Append(match.ToString());
            }

            return stringBuilder.ToString();
        }

        /// <summary>
        /// Black Box fitness tracker status
        /// </summary>
        private void BBFSS()
        {
            fileName = Path.GetFileName(BBFpath);

            //check if process still running
            Process[] processName = Process.GetProcessesByName(fileName.Substring(0, fileName.LastIndexOf('.')));
            if (processName.Length > 0)
            {
                FitnessStatusIcon.Image = Properties.Resources.trafficlight_green;
            }
            else
            {
                FitnessStatusIcon.Image = Properties.Resources.trafficlight_red;
                BBFSStatus = false;
            }
        }

        #endregion PluginProcessCheck

        #region ApplicationStartup

        /// <summary>
        /// Presence sensor file location
        /// </summary>
        private void StartPIR()
        {
            try
            {
                PIRSensorTask.StartInfo.FileName = BBPpath;
                PIRSensorTask.StartInfo.CreateNoWindow = false;
                PIRSensorTask.Start();
            }
            catch (Exception ex)
            {
                BBSensorErrorLog.SaveErrorData(ex.Message.ToString());
            }
        }

        /// <summary>
        /// thermostat file location
        /// </summary>
        private void StartThermostate()
        {
            try
            {
                ThermostatTask.StartInfo.FileName = BBTpath;
                ThermostatTask.StartInfo.CreateNoWindow = false;
                ThermostatTask.Start();
            }
            catch (Exception ex)
            {
                ThermostatStatusIcon.Image = Properties.Resources.trafficlight_red;
                BBSensorErrorLog.SaveErrorData(ex.Message.ToString());
            }
        }

        /// <summary>
        /// Fitness tracker file location
        /// </summary>
        private void StartFitnessTracker()
        {
            try
            {
                FitnessTask.StartInfo.FileName = BBFpath;
                FitnessTask.StartInfo.CreateNoWindow = false;
                FitnessTask.Start();
            }
            catch (Exception ex)
            {
                BBSensorErrorLog.SaveErrorData(ex.Message.ToString());
            }
        }

        #endregion ApplicationStartup

        #region PluginStatusCheck

        /// <summary>
        /// Checking the operation status
        /// </summary>
        private void BBTaskCheck()
        {
            StatusIcon.Image = Properties.Resources.trafficlight_green;

            try
            {
                // Checking the status of the sensors
                if (BBFSStatus && BBPSStatus && BBTSStatus && ScheduleSet)
                {
                    SchedulerIcon.Image = Properties.Resources.trafficlight_green;
                }
                if (BBFSStatus || BBPSStatus || BBTSStatus && ScheduleSet)
                {
                    SchedulerIcon.Image = Properties.Resources.trafficlight_green;
                }
                if (!BBFSStatus && !BBPSStatus && !BBTSStatus && ScheduleSet)
                {
                    SchedulerIcon.Image = Properties.Resources.trafficlight_yellow;
                }
                if (BBFSStatus && BBPSStatus && BBTSStatus && !ScheduleSet)
                {
                    SchedulerIcon.Image = Properties.Resources.trafficlight_yellow;
                }
            }
            catch (Exception ex)
            {
                BBSensorErrorLog.SaveErrorData(ex.Message.ToString());
                SchedulerIcon.Image = Properties.Resources.trafficlight_red;
            }
        }

        /// <summary>
        /// Intervals which status are checked
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StatusChecker_Tick(object sender, EventArgs e)
        {
            TSStatusChecker();
        }

        /// <summary>
        /// Check operation status
        /// </summary>
        private void TSStatusChecker()
        {
            if ((startdate <= today) && (today <= enddate) && ScheduleSet)
            {
                ///Thermostat
                try
                {
                    BBTSS();
                }
                catch (Exception ex)
                {
                    StatusIcon.Image = Properties.Resources.trafficlight_red;
                    BBSensorErrorLog.SaveErrorData(ex.Message.ToString());
                }
                //Fitness tracker
                try
                {
                    BBFSS();
                }
                catch (Exception ex)
                {
                    StatusIcon.Image = Properties.Resources.trafficlight_red;
                    BBSensorErrorLog.SaveErrorData(ex.Message.ToString());
                }
                //Presence sensor
                try
                {
                    BBPSS();

                    BBTaskCheck();
                }
                catch (Exception ex)
                {
                    MotionStatusIcon.Image = Properties.Resources.trafficlight_red;
                    StatusIcon.Image = Properties.Resources.trafficlight_red;
                    BBSensorErrorLog.SaveErrorData(ex.Message.ToString());
                }

                // Update progress bar
                ResearchSpand.Value = (today).Day / startdate.Day;
            }
            if (ResearchSpand.Value == 100)
            {
                BBTaskCheck();
                StatusChecker.Stop();
                MessageBox.Show("You have come to the end of the\n research period, thank you for participating.");
                ScheduleSet = false;
                PIRSensorTask.Kill();
                ThermostatTask.Kill();
                FitnessTask.Kill();
            }

            //Reset status
            BBPSStatus = false;

            //Resetting thermostat data status
            BBTSStatus = false;

            //Resetting Fitness tracker status
            BBFSStatus = false;
        }

        #endregion PluginStatusCheck

        #region ButtonControls

        /// <summary>
        /// Check network status
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BBSmartSetup_Click(object sender, EventArgs e)
        {
            BBAccessPad AccessPad = new BBAccessPad();
            AccessPad.ShowDialog();

            if (BBAccessPad.access)
            {
                ProcessStartInfo proc1 = new ProcessStartInfo();
                string Command;
                proc1.UseShellExecute = true;
                Command = @"netsh wlan show hostednetwork";
                proc1.WorkingDirectory = @"C:\Windows\System32";
                proc1.FileName = @"C:\Windows\System32\cmd.exe";
                proc1.Verb = "runas";
                proc1.Arguments = "/k " + Command;
                proc1.WindowStyle = ProcessWindowStyle.Normal;
                Process.Start(proc1);
            }
        }

        /// <summary>
        /// Open the advance dialog box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BBAdvSetup_Click(object sender, EventArgs e)
        {
            BBAccessPad AccessPad = new BBAccessPad();
            AccessPad.ShowDialog();

            if (BBAccessPad.access)
            {
                StartFitnessTracker();
            }
        }

        /// <summary>
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BBtaskScheduler_Load(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Get and check schedule dates
        /// </summary>
        /// <returns></returns>
        private bool ScheduleChecker()
        {
            ResearchSpand.Value = 0;
            startdate = Convert.ToDateTime(StartDateObj.Value.ToString("MM/dd/yyyy"));
            enddate = Convert.ToDateTime(StopDateObj.Value.ToString("MM/dd/yyyy"));

            if (startdate != null && enddate != null)
            {
                int value = (int)((enddate - startdate).TotalDays);
                ResearchSpand.Maximum = value;
                ResearchSpand.Minimum = 0;

                StatusChecker.Start();
                ScheduleSet = true;
            }
            else
            {
                StatusChecker.Start();
                ScheduleSet = false;
            }

            return ScheduleSet;
        }

        /// <summary>
        /// Restart task scheduler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BBRestart_Click(object sender, EventArgs e)
        {
            BBAccessPad AccessPad = new BBAccessPad();
            AccessPad.ShowDialog();

            if (BBAccessPad.access)
            {
                PIRSensorTask.Kill();
                ThermostatTask.Kill();
                FitnessTask.Kill();
                Thread.Sleep(1000);
                PIRSensorTask.Start();
                Thread.Sleep(500);
                ThermostatTask.Start();
                Thread.Sleep(500);
                FitnessTask.Kill();
            }
        }

        #endregion ButtonControls

        #region DashBoardControls

        /// <summary>
        /// validate selected dates
        /// </summary>
        private void CheckDate()
        {
            DateTime today = DateTime.Today;
            if ((startdate <= today) && (today <= enddate))
            {
                ScheduleSet = ScheduleChecker();

                if (ScheduleSet)
                {
                    ScheduleChecker();
                    StatusChecker.Start();
                    Thread.Sleep(500);
                    StartThermostate();
                    Thread.Sleep(500);
                    StartFitnessTracker();
                    Thread.Sleep(500);
                    StartPIR();
                }
            }
            else
            {
                StopDate = "";
                StartDate = "";
                MessageBox.Show("Schedule date invalid");
            }
        }

        /// <summary>
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void submitBtn_Click(object sender, EventArgs e)
        {
            BBAccessPad AccessPad = new BBAccessPad();
            AccessPad.ShowDialog();
            if (BBAccessPad.access)
            {
                ScheduleChecker();
                CheckDate();
            }
        }

        /// <summary>
        /// power down button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void offBtn_Click(object sender, EventArgs e)
        {
            BBAccessPad AccessPad = new BBAccessPad();
            AccessPad.ShowDialog();

            if (BBAccessPad.access)

            {
                try
                {
                    PIRSensorTask.Kill();
                    ThermostatTask.Kill();
                    FitnessTask.Kill();
                    Thread.Sleep(1000);
                    this.Close();
                }
                catch
                {
                    this.Close();
                }
            }
        }

        /// <summary>
        /// Stop network connection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void networkDisconnect_Click(object sender, EventArgs e)
        {
            BBAccessPad AccessPad = new BBAccessPad();
            AccessPad.ShowDialog();

            if (BBAccessPad.access)
            {
                ProcessStartInfo proc1 = new ProcessStartInfo();
                string Command;
                proc1.UseShellExecute = true;
                Command = @"netsh wlan stop hostednetwork";
                proc1.WorkingDirectory = @"C:\Windows\System32";
                proc1.FileName = @"C:\Windows\System32\cmd.exe";
                proc1.Verb = "runas";
                proc1.Arguments = "/k " + Command;
                proc1.WindowStyle = ProcessWindowStyle.Hidden;
                Process.Start(proc1);
            }
        }

        /// <summary>
        /// Start network connection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void networkSetup_Click(object sender, EventArgs e)
        {
            BBAccessPad AccessPad = new BBAccessPad();
            AccessPad.ShowDialog();

            if (BBAccessPad.access)
            {
                ProcessStartInfo proc1 = new ProcessStartInfo();
                string Command;
                proc1.UseShellExecute = true;
                Command = @"netsh wlan start hostednetwork";
                proc1.WorkingDirectory = @"C:\Windows\System32";
                proc1.FileName = @"C:\Windows\System32\cmd.exe";
                proc1.Verb = "runas";
                proc1.Arguments = "/k " + Command;
                proc1.WindowStyle = ProcessWindowStyle.Hidden;
                Process.Start(proc1);
            }
        }

        /// <summary>
        /// Network connection window
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void adaptorBtn_Click(object sender, EventArgs e)
        {
            BBAccessPad AccessPad = new BBAccessPad();
            AccessPad.ShowDialog();

            if (BBAccessPad.access)
            {
                ProcessStartInfo proc1 = new ProcessStartInfo();
                string Command;
                proc1.UseShellExecute = true;
                Command = @"ncpa.cpl";
                proc1.WorkingDirectory = @"C:\Windows\System32";
                proc1.FileName = @"C:\Windows\System32\cmd.exe";
                proc1.Arguments = "/k " + Command;
                proc1.WindowStyle = ProcessWindowStyle.Hidden;
                Process.Start(proc1);
            }
        }

        /// <summary>
        /// Setup Black Box network
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NetSetup_Click(object sender, EventArgs e)
        {
            BBAccessPad AccessPad = new BBAccessPad();
            AccessPad.ShowDialog();

            if (BBAccessPad.access)
            {
                Process proc = null;
                try
                {
                    string batDir = string.Format(@"NetSetup\");
                    proc = new Process();
                    proc.StartInfo.WorkingDirectory = batDir;
                    proc.StartInfo.FileName = BBNSetup;
                    proc.StartInfo.CreateNoWindow = false;
                    proc.Start();
                    proc.WaitForExit();
                    MessageBox.Show("Bat file executed !!");
                }
                catch (Exception)
                {
                    MessageBox.Show("Error setting up access point", "Configureation Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        #endregion DashBoardControls
    }
}